/******************************************************************************
 * @file     dma.c
 * @brief    DMA firmware functions.
 * @version  V1.6.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "dma.h"




/******************************************************************************
 * @brief      Set handshake with SSP Tx (Memory to Peripheral)
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
void __DMA_SetHandShakeWithSSPTx(DMACH_REGS *DMACHx)
{
    /* Set destination address - SSPDATA */
    DMA_SetDestinationAddr(DMACHx, SSP_BASE + 0x10UL);

    /* Set destination peripheral */
    DMA_SetDestinationPeripheral(DMACHx, DMA_DPER_SSP_TX);

    /* Set Transfer type */
    DMA_SetTransferType(DMACHx, DMA_MEMORY_TO_PERIPHERAL);

    /* Set destination Address mode */
    DMA_SetDestinationAddrMode(DMACHx, DMA_ADDRESS_NO_CHANGE);

    /* Set destination handshaking */
    DMA_SetDestinationHandShake(DMACHx, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
}




/******************************************************************************
 * @brief      Set handshake with SSP Rx (Peripheral to Memory)
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
void __DMA_SetHandShakeWithSSPRx(DMACH_REGS *DMACHx)
{
    /* Set source address - SSPDATA */
    DMA_SetSourceAddr(DMACHx, SSP_BASE + 0x10UL);

    /* Set source peripheral */
    DMA_SetSourcePeripheral(DMACHx, DMA_SPER_SSP_RX);

    /* Set Transfer type */
    DMA_SetTransferType(DMACHx, DMA_PERIPHERAL_TO_MEMORY);

    /* Set source Address mode */
    DMA_SetSourceAddrMode(DMACHx, DMA_ADDRESS_NO_CHANGE);

    /* Set source handshaking */
    DMA_SetSourceHandShake(DMACHx, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
}




/******************************************************************************
 * @brief      Set handshake with UART Tx (Memory to Peripheral)
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
void __DMA_SetHandShakeWithUARTTx(DMACH_REGS *DMACHx)
{
    /* Set destination address - UARTTHR */
    DMA_SetDestinationAddr(DMACHx, UART_BASE);

    /* Set destination peripheral */
    DMA_SetDestinationPeripheral(DMACHx, DMA_DPER_UART_TX);

    /* Set Transfer type */
    DMA_SetTransferType(DMACHx, DMA_MEMORY_TO_PERIPHERAL);

    /* Set destination Address mode */
    DMA_SetDestinationAddrMode(DMACHx, DMA_ADDRESS_NO_CHANGE);

    /* Set destination handshaking */
    DMA_SetDestinationHandShake(DMACHx, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
}




/******************************************************************************
 * @brief      Set handshake with UART Rx (Peripheral to Memory)
 *
 * @param[in]  DMACHx:  Select the DMA channel (DMACH0 ~ DMACH5)
 *
 * @return     none
 *
 ******************************************************************************/
void __DMA_SetHandShakeWithUARTRx(DMACH_REGS *DMACHx)
{
    /* Set source address - UARTRBR */
    DMA_SetSourceAddr(DMACHx, UART_BASE);

    /* Set source peripheral */
    DMA_SetSourcePeripheral(DMACHx, DMA_SPER_UART_RX);

    /* Set Transfer type */
    DMA_SetTransferType(DMACHx, DMA_PERIPHERAL_TO_MEMORY);

    /* Set source Address mode */
    DMA_SetSourceAddrMode(DMACHx, DMA_ADDRESS_NO_CHANGE);

    /* Set source handshaking */
    DMA_SetSourceHandShake(DMACHx, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
}




/******************************************************************************
 * @brief      Set handshake with peripheral
 *
 * @param[in]  DMACHx     :  Select the DMA channel (DMACH0 ~ DMACH5)
 * @param[in]  ePeripheral:  Select the peripheral
 *                           Please go to DMA_PeripheralEnum for details
 *
 * @return     none
 *
 ******************************************************************************/
void DMA_SetHandShakeWithPeripheral(DMACH_REGS *DMACHx, DMA_PeripheralEnum ePeripheral)
{
    switch (ePeripheral)
    {
        case DMA_SPER_UART_RX:
            __DMA_SetHandShakeWithUARTRx(DMACHx);
            break;

        case DMA_DPER_UART_TX:
            __DMA_SetHandShakeWithUARTTx(DMACHx);
            break;

        case DMA_SPER_SSP_RX:
            __DMA_SetHandShakeWithSSPRx(DMACHx);
            break;

        case DMA_DPER_SSP_TX:
            __DMA_SetHandShakeWithSSPTx(DMACHx);
            break;
    }
}


/******************* (C) COPYRIGHT 2021 SPINTROL LIMITED ***** END OF FILE ****/
