/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.6.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc2168.h"
#include<stdio.h>
#include<stdlib.h>

#define                   DEBUG_INFO                        1                         /*debug infomation flag*/
#define                   DataWidth_8bit                    8                         /*data width size of SSP*/
#define                   SSPBaudrate_3M                    3000000                  /*SSP baudrate*/

uint32_t *pu32Src;
uint32_t *pu32Dst;


 /*************************************************************************************************************************
 *
 * @brief       The demo demonstrate hwo to use DMA transfer data from SLAVE SSP RX to UART TX. We use two CHIP to demonstrate
 *              this useage. CHIP 1(SLAVE SSP) received CHIP 2 (MASTER SSP) data, these datum were transfer to UART TX by DMA.
 *
 *              MASTER SSP sent data(0x30, 0x3F31, 0x32, 0x33, coressponding ASCII is 0/?1/2/3) to SLAVE SSP.
 *
 *                                        CHIP 1 (SLAVE SSP)                                 CHIP 2 (MASTER SSP)
 *                              --------------------------------------                 ----------------------------
 *                             |                                      |               |                            |
 *                             |     ---------        ----------------|               |--------------              |
 *                             |    |         |      |   SSP SLAVE    x------CLK------x  SSP MASTER  |             |
 *                             |    |   DMA   |      |   ---------    x------SFRM-----x   ---------  |             |
 *                             |    |         |<-----|--|  RX     |   x------MOSI-----x  |  TX     | |             |
 *                             |    |         |      |   ---------    x------MISO-----x   ---------  |             |
 *                             |    |         |      |                |               |              |             |
 *                             |     -------|-        ----------------|               |--------------              |
 *                             |            |                         |               |                            |
 *                             |            |         ------------    |               |                            |
 *                             |             ------->| UART TX    |-  |               |                            |
 *                             |                      ------------  | |               |                            |
 *                              ------------------------------------|-                 ----------------------------
 *                                                                  |
 *                                                                 \ /
 *                                                           OUTPUT to display
 *
 *              NOTE:
 *                   <1> The communication data rate should be carefully chosen for both the SSP and UART to avoid FIFO 
 *                       overflow or underflow.
 *                   <2> UART bus width is 8bit, so if the data transfered by DMA exceed 8bit, the exceeding bit will be 
 *                       ignored by UART. So the final data in UART is 0x30, 0x31, 0x32, 0x33(coressponding ASCII is 0/1/2/3).
 *                   <3> CHIP 1 and CHIP 2 are all SPC2168 in this demo.
 *
 *************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_44,GPIO44_UART_TXD);
  GPIO_SetPinChannel(GPIO_45,GPIO45_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Set GPIOs as SSP function */
  GPIO_SetPinChannel(GPIO_38,GPIO38_SPI_SCLK);
  GPIO_SetPinChannel(GPIO_39,GPIO39_SPI_SFRM);
  GPIO_SetPinChannel(GPIO_40,GPIO40_SPI_MOSI);
  GPIO_SetPinChannel(GPIO_41,GPIO41_SPI_MISO);

  /* Set Output Strength(for high speed) */
  GPIO_SetOutStrength(GPIO_38, GPIO_OUT_STRENGTH_20MA);
  GPIO_SetOutStrength(GPIO_39, GPIO_OUT_STRENGTH_20MA);
  GPIO_SetOutStrength(GPIO_40, GPIO_OUT_STRENGTH_20MA);
  GPIO_SetOutStrength(GPIO_41, GPIO_OUT_STRENGTH_20MA);

  /* -------------- Step 1 : Config SSP for DMA transmission -------------- */
  /*
   * Init SSP Baud Rate, Frame Format
   * 1) Set master mode
   * 2) Set frame data 8-bits width
   * 3) Set transfer speed 30Mbps
   * 4) Set pin SFRM active low
   * 5) Set SCLK Polarity: SCLK is low when bus idle
   * 6) Set SCLK Phase: sample data on SCLK rising edge
   */
  SSP_Init(SSP, SSP_SLAVE_MODE, DataWidth_8bit, SSPBaudrate_3M, SSP_FRM_ACTIVE_LOW, 
                          SSP_CLK_IDLE_LOW,SSP_SEND_ON_FALLING_EDGE);
                          
  /* Disable FIFO Packed Mode */
  SSP_DisableFIFOPackMode(SSP);
  
  /* Enable SSP RX DMA transfer */
  SSP_EnableDMAReceiveService(SSP);
  
  /* Set the SSP RX threshold(uint is word) to triger a CPU INT, set to 0(1 word) make sure all data can be transfered */
  SSP_SetRxFIFOTriggerThreshold(SSP,0x0);
  
  /* uncoment this code to check whether SLAVE SSP received MASTER SSP data or not */
//  SSP_EnableRxFullInt(SSP);

  /* Enable SSP */
  SSP_Enable(SSP);
  /* -------------- Step 1 OVER -------------- */
  
  /* -------------- Step 2 : Config UART for DMA transmission -------------- */
  /* 
   * NOTE:
   *      UART bus width is set to 8bit, that is to say, if DMA transfer a data exceed 8bit to UART TX FIFO, the bit exceed
   *      8bit will be ignored.
   */
  /* Disable UART */
  UART_Disable(UART);

  /* Config FIFO */
  UART->UARTIIR_UARTFCR.UARTFCR.all = UARTFCR_ALL_TRFIFOE_FIFO_ENABLE |       /*                                          */
                                      UARTFCR_ALL_CLRRF_CLEAR_RX_FIFO |       /*                                          */
                                      UARTFCR_ALL_CLRTF_CLEAR_TX_FIFO |       /*                                          */
                                      UARTFCR_ALL_TXTH_TXFIFO_EMPTY |         /* Interrupt/DMA request when FIFO is empty */
                                      UARTFCR_ALL_BUS32_8BIT |                /*                                          */
                                      UARTFCR_ALL_RXTH_RX_FIFO_THRESHOLD_1;   /* 1 byte or more in FIFO causes interrupt */

  /* Enable UART DMA */
  UART_EnableDMAService(UART);
  
  /* Enable UART */
  UART_Enable(UART);
  /* -------------- Step 2 OVER -------------- */
  
  /* -------------- Step 3 : Config DMA for the transmission from SSP RX to UART TX -------------- */
  pu32Src = (uint32_t *)(&SSP->SSPDATA.all);                                  /* Set source data address */
  pu32Dst = (uint32_t *)(&UART->UARTRBR_UARTTHR_UARTDLL.UARTTHR.all);         /* Set the distination address */
  
  /* Enable DMA */
  DMA_Enable();
  
  /* Source address */
  DMA_SetSourceAddr(DMACH0, (uint32_t)pu32Src);
  /* Destination address */
  DMA_SetDestinationAddr(DMACH0, (uint32_t)pu32Dst);
  
  /* Transfer type */
  DMA_SetTransferType(DMACH0, DMACHCTL0_BIT_TT_PER_TO_PER);
  
  /* Source address increment */
  DMA_SetSourceAddrMode(DMACH0, DMA_ADDRESS_NO_CHANGE);
  /* Destination address increment */
  DMA_SetDestinationAddrMode(DMACH0, DMA_ADDRESS_NO_CHANGE);
  
  /* Source burst transfer length - 1 words = UART RXFIFO Threshold */
  DMA_SetSourceBurstLen(DMACH0, DMACHCTL0_BIT_SSIZE_1_WORD);
  /* Set destination burst transfer length as the same as Source */
  DMA_SetDestinationBurstLen(DMACH0, DMACHCTL0_BIT_DSIZE_1_WORD);
  
  /* Block transfer size, Master SSP sent 4 entery to SSP RX, so DMA need to tansfer 4 word data(DMA bus is 32 width) */
  DMA_SetBlockTransferSize(DMACH0, 0x4);
  
  /* Set source handshaking select and handshaking interface polarity */
  DMA_SetSourceHandShake(DMACH0, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
  /* Set destination handshaking select and handshaking interface polarity */
  DMA_SetDestinationHandShake(DMACH0, DMA_HANDSHAKE_BY_HARDWARE, DMA_HANDSHAKE_POL_ACTIVE_HIGH);
  
  /* Source peripheral */
  DMA_SetSourcePeripheral(DMACH0, DMACHCTL3_BIT_SPER_SSP_RX);
  /* destination peripheral */
  DMA_SetDestinationPeripheral(DMACH0, DMACHCTL3_BIT_DPER_UART_TX);
  
  /* Enable channel 0 interruput */
  DMA_EnableChannelInt(DMACH0);
  /* Enable Channel 0 transfer complete interrupt */
  DMA_EnableTransferCompleteInt(DMA_CH0);
  
  /* Enable Channel 0 */
  DMA_EnableChannelTransfer(DMA_CH0);
  /* -------------- Step 2 OVER -------------- */
  
  /* Enable global INT for DMA */
  NVIC_EnableIRQ(DMAC_IRQn);
  /* Enable global INT for SSP */
  NVIC_EnableIRQ(SSP_IRQn);

  while(1)
  {

  }
}

void DMAC_IRQHandler(void)
{
  if(DMA_GetErrorGlobalIntFlag() == 1)
  {
    printf("DMA Channel Error!\n");
  }
  
  if(DMA_GetTransferCompleteGlobalIntFlag() == 1)
  {
    printf("Channel transfer complete!\n");
  }
  
  if(DMA_GetTransferCompleteIntFlag(DMA_CH0) == 1)
  {
    printf("Channel 0 transfer complete!\n");
    
    DMA_ClearTransferCompleteInt(DMA_CH0);
  }
}

void SSP_IRQHandler(void)
{
    /* If you uncoment the 'SSP_EnableRxFullInt(SSP);', if SLAVE SSP had received MASTER SSP data, function will enter this function */
    printf("SLAVE SSP had received data from MASTER SSP\n");
}
